@@ -120,6 +120,13 @@ static gboolean bee_irc_user_status(bee_t *bee, bee_user_t *bu, bee_user_t *old)
 		iu->flags |= IRC_USER_AWAY;
 	}
 
+	/* return early if nothing changed */
+	if ((bu->flags == old->flags) &&
+	    (g_strcmp0(bu->status, old->status) == 0) &&
+	    (g_strcmp0(bu->status_msg, old->status_msg) == 0)) {
+		return TRUE;
+	}
+
 	if ((bu->flags & BEE_USER_ONLINE) != (old->flags & BEE_USER_ONLINE)) {
 		if (bu->flags & BEE_USER_ONLINE) {
 			if (g_hash_table_lookup(irc->watches, iu->key)) {
@@ -143,21 +150,11 @@ static gboolean bee_irc_user_status(bee_t *bee, bee_user_t *bu, bee_user_t *old)
 		}
 	}
 
-	/* Reset this one since the info may have changed. */
 	iu->away_reply_timeout = 0;
 
 	bee_irc_channel_update(irc, NULL, iu);
 
-	/* If away-notify enabled, send status updates when:
-	 * Away or Online state changes
-	 * Status changes (e.g. "Away" to "Mobile")
-	 * Status message changes
-	 */
-	if ((irc->caps & CAP_AWAY_NOTIFY) &&
-	    ((bu->flags & BEE_USER_AWAY) != (old->flags & BEE_USER_AWAY) ||
-	     (bu->flags & BEE_USER_ONLINE) != (old->flags & BEE_USER_ONLINE) ||
-	     (g_strcmp0(bu->status, old->status) != 0) ||
-	     (g_strcmp0(bu->status_msg, old->status_msg) != 0))) {
+	if (irc->caps & CAP_AWAY_NOTIFY) {
 		irc_send_away_notify(iu);
 	}
 
@@ -180,10 +177,14 @@ void bee_irc_channel_update(irc_t *irc, irc_channel_t *ic, irc_user_t *iu)
 		return;
 	}
 	if (iu == NULL) {
-		for (l = irc->users; l; l = l->next) {
-			iu = l->data;
+		GHashTableIter iter;
+		gpointer itervalue;
+		g_hash_table_iter_init(&iter, irc->nick_user_hash);
+
+		while (g_hash_table_iter_next(&iter, NULL, &itervalue)) {
+			iu = itervalue;
 			if (iu->bu) {
-				bee_irc_channel_update(irc, ic, l->data);
+				bee_irc_channel_update(irc, ic, iu);
 			}
 		}
 		return;
@@ -227,7 +228,9 @@ static gboolean bee_irc_user_msg(bee_t *bee, bee_user_t *bu, const char *msg_, g
 	char *message_type = "PRIVMSG";
 	GSList *l;
 
-	if (sent_at > 0 && set_getbool(&irc->b->set, "display_timestamps")) {
+	if (sent_at > 0 &&
+	    !(irc->caps & CAP_SERVER_TIME) &&
+	    set_getbool(&irc->b->set, "display_timestamps")) {
 		ts = irc_format_timestamp(irc, sent_at);
 	}
 
@@ -300,7 +303,7 @@ static gboolean bee_irc_user_msg(bee_t *bee, bee_user_t *bu, const char *msg_, g
 	}
 
 	wrapped = word_wrap(msg, IRC_WORD_WRAP);
-	irc_send_msg(src_iu, message_type, dst, wrapped, prefix);
+	irc_send_msg_ts(src_iu, message_type, dst, wrapped, prefix, sent_at);
 	g_free(wrapped);
 
 cleanup:
@@ -445,6 +448,35 @@ void bee_irc_user_nick_reset(irc_user_t *iu)
 
 }
 
+#define PASTEBUF_LONG_SPACELESS_LINE_LENGTH 350
+
+/* Returns FALSE if the last line of the message is near the typical irc length
+ * limit and the message has no spaces, indicating that it's probably desirable
+ * to join messages without the newline.
+ *
+ * The main use case for this is pasting long URLs and not breaking them */
+static gboolean bee_irc_pastebuf_should_start_with_newline(const char *msg)
+{
+	int i;
+	const char *last_line = strrchr(msg, '\n') ? : msg;
+
+	if (*last_line == '\n') {
+		last_line++;
+	}
+
+	for (i = 0; last_line[i]; i++) {
+		if (g_ascii_isspace(last_line[i])) {
+			return TRUE;
+		}
+	}
+
+	if (i < PASTEBUF_LONG_SPACELESS_LINE_LENGTH) {
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /* IRC->IM calls */
 
 static gboolean bee_irc_user_privmsg_cb(gpointer data, gint fd, b_input_condition cond);
@@ -469,7 +501,10 @@ static gboolean bee_irc_user_privmsg(irc_user_t *iu, const char *msg)
 		iu->pastebuf = g_string_new(msg);
 	} else {
 		b_event_remove(iu->pastebuf_timer);
-		g_string_append_printf(iu->pastebuf, "\n%s", msg);
+		if (bee_irc_pastebuf_should_start_with_newline(iu->pastebuf->str)) {
+			g_string_append_c(iu->pastebuf, '\n');
+		}
+		g_string_append(iu->pastebuf, msg);
 	}
 
 	if (set_getbool(&iu->irc->b->set, "paste_buffer")) {
@@ -665,12 +700,14 @@ static gboolean bee_irc_chat_msg(bee_t *bee, struct groupchat *c, bee_user_t *bu
 		return FALSE;
 	}
 
-	if (sent_at > 0 && set_getbool(&bee->set, "display_timestamps")) {
+	if (sent_at > 0 &&
+	    !(irc->caps & CAP_SERVER_TIME) &&
+	    set_getbool(&bee->set, "display_timestamps")) {
 		ts = irc_format_timestamp(irc, sent_at);
 	}
 
 	wrapped = word_wrap(msg, IRC_WORD_WRAP);
-	irc_send_msg(iu, "PRIVMSG", ic->name, wrapped, ts);
+	irc_send_msg_ts(iu, "PRIVMSG", ic->name, wrapped, ts, sent_at);
 	g_free(ts);
 	g_free(wrapped);
 
