@@ -363,11 +363,21 @@ void irc_send_who(irc_t *irc, GSList *l, const char *channel)
 }
 
 void irc_send_msg(irc_user_t *iu, const char *type, const char *dst, const char *msg, const char *prefix)
+{
+	irc_send_msg_ts(iu, type, dst, msg, prefix, 0);
+}
+
+void irc_send_msg_ts(irc_user_t *iu, const char *type, const char *dst, const char *msg, const char *prefix, time_t ts)
 {
 	char last = 0;
 	const char *s = msg, *line = msg;
+	char *tags = NULL;
 	char raw_msg[strlen(msg) + 1024];
 
+	if (!(iu->irc->caps & CAP_SERVER_TIME)) {
+		ts = 0;
+	}
+
 	while (!last) {
 		if (*s == '\r' && *(s + 1) == '\n') {
 			s++;
@@ -378,19 +388,25 @@ void irc_send_msg(irc_user_t *iu, const char *type, const char *dst, const char
 			last = s[0] == 0;
 		}
 		if (*s == 0 || *s == '\n') {
+			if (ts) {
+				tags = irc_format_servertime(iu->irc, ts);
+			}
 			if (g_strncasecmp(line, "/me ", 4) == 0 && (!prefix || !*prefix) &&
 			    g_strcasecmp(type, "PRIVMSG") == 0) {
 				strcpy(raw_msg, "\001ACTION ");
 				strncat(raw_msg, line + 4, s - line - 4);
 				strcat(raw_msg, "\001");
-				irc_send_msg_raw(iu, type, dst, raw_msg);
+				irc_send_msg_raw_tags(iu, type, dst, tags, raw_msg);
 			} else {
 				*raw_msg = '\0';
 				if (prefix && *prefix) {
 					strcpy(raw_msg, prefix);
 				}
 				strncat(raw_msg, line, s - line);
-				irc_send_msg_raw(iu, type, dst, raw_msg);
+				irc_send_msg_raw_tags(iu, type, dst, tags, raw_msg);
+			}
+			if (ts) {
+				g_free(tags);
 			}
 			line = s + 1;
 		}
@@ -400,8 +416,13 @@ void irc_send_msg(irc_user_t *iu, const char *type, const char *dst, const char
 
 void irc_send_msg_raw(irc_user_t *iu, const char *type, const char *dst, const char *msg)
 {
-	irc_write(iu->irc, ":%s!%s@%s %s %s :%s",
-	          iu->nick, iu->user, iu->host, type, dst, msg && *msg ? msg : " ");
+	irc_send_msg_raw_tags(iu, type, dst, NULL, msg);
+}
+
+void irc_send_msg_raw_tags(irc_user_t *iu, const char *type, const char *dst, const char* tags, const char *msg)
+{
+	irc_write(iu->irc, "%s%s:%s!%s@%s %s %s :%s",
+	          tags ? tags : "", tags ? " " : "", iu->nick, iu->user, iu->host, type, dst, msg && *msg ? msg : " ");
 }
 
 void irc_send_msg_f(irc_user_t *iu, const char *type, const char *dst, const char *format, ...)
