@@ -1,4 +1,4 @@
-/********************************************************************\
+  /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
   * Copyright 2002-2012 Wilmer van der Gaast and others                *
@@ -334,51 +334,6 @@ char *strip_newlines(char *source)
 	return source;
 }
 
-/* Wrap an IPv4 address into IPv6 space. Not thread-safe... */
-char *ipv6_wrap(char *src)
-{
-	static char dst[64];
-	int i;
-
-	for (i = 0; src[i]; i++) {
-		if ((src[i] < '0' || src[i] > '9') && src[i] != '.') {
-			break;
-		}
-	}
-
-	/* Hmm, it's not even an IP... */
-	if (src[i]) {
-		return src;
-	}
-
-	g_snprintf(dst, sizeof(dst), "::ffff:%s", src);
-
-	return dst;
-}
-
-/* Unwrap an IPv4 address into IPv6 space. Thread-safe, because it's very simple. :-) */
-char *ipv6_unwrap(char *src)
-{
-	int i;
-
-	if (g_strncasecmp(src, "::ffff:", 7) != 0) {
-		return src;
-	}
-
-	for (i = 7; src[i]; i++) {
-		if ((src[i] < '0' || src[i] > '9') && src[i] != '.') {
-			break;
-		}
-	}
-
-	/* Hmm, it's not even an IP... */
-	if (src[i]) {
-		return src;
-	}
-
-	return (src + 7);
-}
-
 /* Convert from one charset to another.
 
    from_cs, to_cs: Source and destination charsets
@@ -818,3 +773,22 @@ char *str_pad_and_truncate(const char *string, long char_len, const char *ellips
 		return g_strdup(string);
 	}
 }
+
+/* copied from irssi's misc.c, by timo sirainen */
+int b_istr_equal(gconstpointer v, gconstpointer v2)
+{
+	return g_ascii_strcasecmp((const char *) v, (const char *) v2) == 0;
+}
+
+/* copied from irssi's misc.c, by lemonboy */
+guint b_istr_hash(gconstpointer v)
+{
+	const signed char *p;
+	guint32 h = 5381;
+
+	for (p = v; *p != '\0'; p++) {
+		h = (h << 5) + h + g_ascii_toupper(*p);
+	}
+
+	return h;
+}
