@@ -322,7 +322,7 @@ static gboolean ssl_connected(gpointer data, gint source, b_input_condition cond
 #endif
 	gnutls_set_default_priority(conn->session);
 	gnutls_credentials_set(conn->session, GNUTLS_CRD_CERTIFICATE, xcred);
-	if (conn->hostname && !g_ascii_isdigit(conn->hostname[0])) {
+	if (conn->hostname && !g_hostname_is_ip_address(conn->hostname)) {
 		gnutls_server_name_set(conn->session, GNUTLS_NAME_DNS,
 		                       conn->hostname, strlen(conn->hostname));
 	}
