@@ -877,13 +877,13 @@ void op_handle_smp_event(void *opdata, OtrlSMPEvent ev, ConnContext *ctx,
 	switch (ev) {
 	case OTRL_SMPEVENT_ASK_FOR_SECRET:
 		irc_rootmsg(irc, "smp: initiated by %s"
-		            " - respond with \x02otr smp %s <secret>\x02",
+		            " - respond with \x02otr smp %s \"<secret>\"\x02",
 		            u->nick, u->nick);
 		break;
 	case OTRL_SMPEVENT_ASK_FOR_ANSWER:
 		irc_rootmsg(irc, "smp: initiated by %s with question: \x02\"%s\"\x02", u->nick,
 		            question);
-		irc_rootmsg(irc, "smp: respond with \x02otr smp %s <answer>\x02",
+		irc_rootmsg(irc, "smp: respond with \x02otr smp %s \"<answer>\"\x02",
 		            u->nick);
 		break;
 	case OTRL_SMPEVENT_CHEATED:
@@ -1058,10 +1058,6 @@ void cmd_otr_connect(irc_t *irc, char **args)
 		irc_rootmsg(irc, "%s: unknown user", args[1]);
 		return;
 	}
-	if (!(u->bu->flags & BEE_USER_ONLINE)) {
-		irc_rootmsg(irc, "%s is offline", args[1]);
-		return;
-	}
 
 	/* passing this through the filter so it goes through libotr which
 	 * will replace the simple query string with a proper one */
@@ -1436,10 +1432,6 @@ void otr_smp_or_smpq(irc_t *irc, const char *nick, const char *question,
 		irc_rootmsg(irc, "%s: unknown user", nick);
 		return;
 	}
-	if (!(u->bu->flags & BEE_USER_ONLINE)) {
-		irc_rootmsg(irc, "%s is offline", nick);
-		return;
-	}
 
 	ctx = otrl_context_find(irc->otr->us, u->bu->handle,
 	                        u->bu->ic->acc->user, u->bu->ic->acc->prpl->name, instag, 0, NULL, NULL, NULL);
