@@ -239,11 +239,6 @@ char *explain_unknown_protocol(const char *name)
 	}
 
 #else
-	if (strcmp(name, "aim") == 0 || strcmp(name, "icq") == 0) {
-		return g_strdup("This account uses libpurple specific aliases for oscar. "
-		                "Re-add the account with `account add oscar ...'");
-	}
-
 	extramsg = "If this is a libpurple plugin, you might need to install bitlbee-libpurple instead.";
 #endif
 	return g_strconcat("The protocol plugin is not installed or could not be loaded. "
@@ -253,20 +248,10 @@ char *explain_unknown_protocol(const char *name)
 
 void nogaim_init()
 {
-	extern void msn_initmodule();
-	extern void oscar_initmodule();
 	extern void jabber_initmodule();
 	extern void twitter_initmodule();
 	extern void purple_initmodule();
 
-#ifdef WITH_MSN
-	msn_initmodule();
-#endif
-
-#ifdef WITH_OSCAR
-	oscar_initmodule();
-#endif
-
 #ifdef WITH_JABBER
 	jabber_initmodule();
 #endif
@@ -302,6 +287,8 @@ GSList *get_connections()
 struct im_connection *imcb_new(account_t *acc)
 {
 	struct im_connection *ic;
+	GHashFunc fn_hash = NULL;
+	GEqualFunc fn_equal = NULL;
 
 	ic = g_new0(struct im_connection, 1);
 
@@ -309,6 +296,20 @@ struct im_connection *imcb_new(account_t *acc)
 	ic->acc = acc;
 	acc->ic = ic;
 
+	/* figure out if we have hashing functions compatible with handle_cmp */
+	if (acc->prpl->handle_cmp == g_ascii_strcasecmp) {
+		fn_hash = b_istr_hash;
+		fn_equal = b_istr_equal;
+	} else if (acc->prpl->handle_cmp == g_strcmp0 || acc->prpl->handle_cmp == strcmp) {
+		fn_hash = g_str_hash;
+		fn_equal = g_str_equal;
+	}
+
+	/* only create the hash table if we found them */
+	if (fn_hash && fn_equal) {
+		ic->bee_users = g_hash_table_new_full(fn_hash, fn_equal, NULL, NULL);
+	}
+
 	connections = g_slist_append(connections, ic);
 
 	return(ic);
@@ -326,6 +327,10 @@ void imc_free(struct im_connection *ic)
 		}
 	}
 
+	if (ic->bee_users) {
+		g_hash_table_destroy(ic->bee_users);
+	}
+
 	connections = g_slist_remove(connections, ic);
 	g_free(ic);
 }
@@ -767,7 +772,7 @@ int imc_away_send_update(struct im_connection *ic)
 		}
 		msg = ic->acc->flags & ACC_FLAG_AWAY_MESSAGE ? away : NULL;
 		away = imc_away_state_find(m, away, &msg) ? :
-		       (imc_away_state_find(m, "away", &msg) ? : m->data);
+		       (imc_away_state_find(m, "away", NULL) ? : m->data);
 	} else if (ic->acc->flags & ACC_FLAG_STATUS_MESSAGE) {
 		away = NULL;
 		msg = set_getstr(&ic->acc->set, "status") ?
@@ -801,7 +806,7 @@ static char *imc_away_state_find(GList *gcm, char *away, char **message)
 			/* At least the Yahoo! module works better if message
 			   contains no data unless it adds something to what
 			   we have in state already. */
-			if (strlen(m->data) == strlen(away)) {
+			if (message && strlen(m->data) == strlen(away)) {
 				*message = NULL;
 			}
 
@@ -827,7 +832,7 @@ static char *imc_away_state_find(GList *gcm, char *away, char **message)
 		for (j = 0; imc_away_alias_list[i][j]; j++) {
 			for (m = gcm; m; m = m->next) {
 				if (g_strcasecmp(imc_away_alias_list[i][j], m->data) == 0) {
-					if (!keep_message) {
+					if (!keep_message && message) {
 						*message = NULL;
 					}
 
