@@ -30,6 +30,10 @@
 #include <glib.h>
 #include <purple.h>
 
+#if !PURPLE_VERSION_CHECK(2, 12, 0)
+#define PURPLE_MESSAGE_REMOTE_SEND 0x10000
+#endif
+
 GSList *purple_connections;
 
 /* This makes me VERY sad... :-( But some libpurple callbacks come in without
@@ -102,17 +106,6 @@ static gboolean purple_menu_cmp(const char *a, const char *b)
 	return (*a == '\0' && *b == '\0');
 }
 
-static char *purple_get_account_prpl_id(account_t *acc)
-{
-	/* "oscar" is how non-purple bitlbee calls it,
-	 * and it might be icq or aim, depending on the username */
-	if (g_strcmp0(acc->prpl->name, "oscar") == 0) {
-		return (g_ascii_isdigit(acc->user[0])) ? "prpl-icq" : "prpl-aim";
-	}
-
-	return acc->prpl->data;
-}
-
 static gboolean purple_account_should_set_nick(account_t *acc)
 {
 	/* whitelist of protocols that tend to have numeric or meaningless usernames, and should
@@ -122,7 +115,6 @@ static gboolean purple_account_should_set_nick(account_t *acc)
 	char *whitelist[] = {
 		"prpl-hangouts",
 		"prpl-eionrobb-funyahoo-plusplus",
-		"prpl-icq",
 		"prpl-line",
 		NULL,
 	};
@@ -139,7 +131,7 @@ static gboolean purple_account_should_set_nick(account_t *acc)
 
 static void purple_init(account_t *acc)
 {
-	char *prpl_id = purple_get_account_prpl_id(acc);
+	char *prpl_id = acc->prpl->data;
 	PurplePlugin *prpl = purple_plugins_find_with_id(prpl_id);
 	PurplePluginProtocolInfo *pi = prpl->info->extra_info;
 	PurpleAccount *pa;
@@ -394,7 +386,7 @@ static void purple_login(account_t *acc)
 	purple_connections = g_slist_prepend(purple_connections, ic);
 
 	ic->proto_data = pd = g_new0(struct purple_data, 1);
-	pd->account = purple_account_new(acc->user, purple_get_account_prpl_id(acc));
+	pd->account = purple_account_new(acc->user, acc->prpl->data);
 	pd->input_requests = g_hash_table_new_full(g_direct_hash, g_direct_equal,
 	                                           NULL, g_free);
 	pd->next_request_id = 0;
@@ -1203,7 +1195,10 @@ static void handle_conv_msg(PurpleConversation *conv, const char *who, const cha
  * Those are safe to display. The rest (with just _SEND) may be echoes. */
 static void prplcb_conv_msg(PurpleConversation *conv, const char *who, const char *message, PurpleMessageFlags flags, time_t mtime)
 {
-	if ((!(flags & PURPLE_MESSAGE_SEND)) || (flags & PURPLE_MESSAGE_DELAYED)) {
+	if ((!(flags & PURPLE_MESSAGE_SEND)) ||
+	    (flags & PURPLE_MESSAGE_DELAYED) ||
+	    (flags & PURPLE_MESSAGE_REMOTE_SEND)
+	) {
 		handle_conv_msg(conv, who, message, (flags & PURPLE_MESSAGE_SEND) ? OPT_SELFMESSAGE : 0, mtime);
 	}
 }
@@ -1765,6 +1760,15 @@ static PurpleAccountUiOps bee_account_uiops =
 	NULL,                              /* close_account_request */
 };
 
+static void *prplcb_bitlbee_set_account_password(PurpleAccount *account, char *password)
+{
+	struct im_connection *ic = purple_ic_by_pa(account);
+
+	set_setstr(&ic->acc->set, "password", password ? password : "");
+
+	return GINT_TO_POINTER(TRUE);
+}
+
 extern PurpleXferUiOps bee_xfer_uiops;
 
 static void purple_ui_init()
@@ -1794,6 +1798,7 @@ void purple_initmodule()
 	GList *prots;
 	GString *help;
 	char *dir;
+	gboolean debug_enabled = !!getenv("BITLBEE_DEBUG");
 
 	if (purple_get_core() != NULL) {
 		log_message(LOGLVL_ERROR, "libpurple already initialized. "
@@ -1812,7 +1817,7 @@ void purple_initmodule()
 	purple_plugins_add_search_path(dir);
 	g_free(dir);
 
-	purple_debug_set_enabled(FALSE);
+	purple_debug_set_enabled(debug_enabled);
 	purple_core_set_ui_ops(&bee_core_uiops);
 	purple_eventloop_set_ui_ops(&glib_eventloops);
 	if (!purple_core_init("BitlBee")) {
@@ -1820,6 +1825,7 @@ void purple_initmodule()
 		fprintf(stderr, "libpurple initialization failed.\n");
 		abort();
 	}
+	purple_debug_set_enabled(FALSE);
 
 	if (proxytype != PROXY_NONE) {
 		PurpleProxyInfo *pi = purple_global_proxy_get_info();
@@ -1852,6 +1858,28 @@ void purple_initmodule()
 	purple_signal_connect(purple_conversations_get_handle(), "buddy-typing-stopped",
 	                      &funcs, PURPLE_CALLBACK(prplcb_buddy_typing), NULL);
 
+	/* "bitlbee-set-account-password" signal:
+	 * Replacement API for purple_account_set_password() to be called by
+	 * prpls that wish to store updated passwords or oauth tokens, since
+	 * our password storage doesn't get notified of calls to
+	 * purple_account_set_password().
+	 *
+	 * When used with purple_signal_emit_return_1() returns:
+	 *  - GINT_TO_POINTER(TRUE) if implemented by this version of bitlbee
+	 *  - NULL otherwise.
+	 *
+	 * Originally made for the hangouts plugin.
+	 */
+
+	purple_signal_register(purple_accounts_get_handle(), "bitlbee-set-account-password",
+	                       purple_marshal_POINTER__POINTER_POINTER,
+	                       purple_value_new(PURPLE_TYPE_BOOLEAN), 2,
+	                       purple_value_new(PURPLE_TYPE_SUBTYPE, PURPLE_SUBTYPE_ACCOUNT),
+	                       purple_value_new(PURPLE_TYPE_STRING));
+
+	purple_signal_connect(purple_accounts_get_handle(), "bitlbee-set-account-password",
+	                      &funcs, PURPLE_CALLBACK(prplcb_bitlbee_set_account_password), NULL);
+
 	memset(&funcs, 0, sizeof(funcs));
 	funcs.login = purple_login;
 	funcs.init = purple_init;
@@ -1916,16 +1944,6 @@ void purple_initmodule()
 
 		g_string_append_printf(help, "\n* %s (%s)", ret->name, prot->info->name);
 
-		/* libpurple doesn't define a protocol called OSCAR, but we
-		   need it to be compatible with normal BitlBee. */
-		if (g_strcasecmp(prot->info->id, "prpl-aim") == 0) {
-			ret = g_memdup(&funcs, sizeof(funcs));
-			ret->name = "oscar";
-			/* purple_get_account_prpl_id() determines the actual protocol ID (icq/aim) */
-			ret->data = NULL;
-			register_protocol(ret);
-		}
-
 		info = g_new0(struct plugin_info, 1);
 		info->abiver = BITLBEE_ABI_VERSION_CODE;
 		info->name = ret->name;
