@@ -359,25 +359,11 @@ static const struct oauth_service twitter_oauth = {
 	.consumer_secret = "FCxqcr0pXKzsF9ajmP57S3VQ8V6Drk4o2QYtqMcOszo",
 };
 
-static const struct oauth_service identica_oauth = {
-	"https://identi.ca/api/oauth/request_token",
-	"https://identi.ca/api/oauth/access_token",
-	"https://identi.ca/api/oauth/authorize",
-	.consumer_key = "e147ff789fcbd8a5a07963afbb43f9da",
-	.consumer_secret = "c596267f277457ec0ce1ab7bb788d828",
-};
-
 static gboolean twitter_oauth_callback(struct oauth_info *info);
 
 static const struct oauth_service *get_oauth_service(struct im_connection *ic)
 {
-	struct twitter_data *td = ic->proto_data;
-
-	if (strstr(td->url_host, "identi.ca")) {
-		return &identica_oauth;
-	} else {
-		return &twitter_oauth;
-	}
+	return &twitter_oauth;
 
 	/* Could add more services, or allow configuring your own base URL +
 	   API keys. */
@@ -390,9 +376,8 @@ static void twitter_oauth_start(struct im_connection *ic)
 
 	imcb_log(ic, "Requesting OAuth request token");
 
-	if (!strstr(url, "twitter.com") && !strstr(url, "identi.ca")) {
-		imcb_log(ic, "Warning: OAuth only works with identi.ca and "
-		         "Twitter.");
+	if (!strstr(url, "twitter.com")) {
+		imcb_log(ic, "Warning: OAuth only works with Twitter.");
 	}
 
 	td->oauth_info = oauth_request_token(get_oauth_service(ic), twitter_oauth_callback, ic);
@@ -539,8 +524,8 @@ static void twitter_init(account_t * acc)
 		def_url = TWITTER_API_URL;
 		def_tul = "23";
 		def_mentions = "true";
-	} else {                /* if( strcmp( acc->prpl->name, "identica" ) == 0 ) */
-		def_url = IDENTICA_API_URL;
+	} else {
+		def_url = "";
 		def_tul = "0";
 		def_mentions = "false";
 	}
@@ -557,7 +542,7 @@ static void twitter_init(account_t * acc)
 
 	s = set_add(&acc->set, "fetch_mentions", def_mentions, set_eval_bool, acc);
 
-	s = set_add(&acc->set, "message_length", "140", set_eval_int, acc);
+	s = set_add(&acc->set, "message_length", "280", set_eval_int, acc);
 
 	s = set_add(&acc->set, "target_url_length", def_tul, set_eval_int, acc);
 
@@ -575,8 +560,11 @@ static void twitter_init(account_t * acc)
 	s = set_add(&acc->set, "_last_tweet", "0", NULL, acc);
 	s->flags |= SET_HIDDEN | SET_NOSAVE;
 
+	s = set_add(&acc->set, "in_korea", "false", set_eval_bool, acc);
+	s->flags |= SET_HIDDEN;
+
 	if (strcmp(acc->prpl->name, "twitter") == 0) {
-		s = set_add(&acc->set, "stream", "true", set_eval_bool, acc);
+		s = set_add(&acc->set, "stream", "false", set_eval_bool, acc);
 		s->flags |= ACC_SET_OFFLINE_ONLY;
 	}
 }
