@@ -307,7 +307,7 @@ static gboolean twitter_xt_get_friends_id_list(json_value *node, struct twitter_
 		}
 
 		txl->list = g_slist_prepend(txl->list,
-		                            g_strdup_printf("%" PRIu64, c->u.array.values[i]->u.integer));
+		                            g_strdup_printf("%" PRId64, c->u.array.values[i]->u.integer));
 	}
 
 	c = json_o_get(node, "next_cursor");
@@ -440,7 +440,7 @@ static void twitter_http_get_noretweets_ids(struct http_request *req)
 
 	txl = g_new0(struct twitter_xml_list, 1);
 	txl->list = td->noretweets_ids;
-	
+
 	// Process the retweet ids
 	txl->type = TXL_ID;
 	if (parsed->type == json_array) {
@@ -451,7 +451,7 @@ static void twitter_http_get_noretweets_ids(struct http_request *req)
 				continue;
 			}
 			txl->list = g_slist_prepend(txl->list,
-			                            g_strdup_printf("%"PRIu64, c->u.integer));
+			                            g_strdup_printf("%" PRId64, c->u.integer));
 		}
 	}
 
@@ -703,8 +703,9 @@ static struct twitter_xml_status *twitter_xt_get_dm(const json_value *node)
 
 static void expand_entities(char **text, const json_value *node, const json_value *extended_node)
 {
-	json_value *entities, *extended_entities, *quoted;
-	char *quote_url = NULL, *quote_text = NULL;
+	json_value *entities, *extended_entities, *quoted, *quoted_permalink;
+	char *quote_url = NULL, *quote_text = NULL, *quote_kort = NULL;
+	gboolean quote_used = FALSE;
 
 	if (!((entities = json_o_get(node, "entities")) && entities->type == json_object))
 		return;
@@ -715,6 +716,9 @@ static void expand_entities(char **text, const json_value *node, const json_valu
 		struct twitter_xml_status *txs = twitter_xt_get_status(quoted);
 		quote_text = g_strdup_printf("@%s: %s", txs->user->screen_name, txs->text);
 		quote_url = g_strdup_printf("%s/status/%" G_GUINT64_FORMAT, txs->user->screen_name, txs->id);
+		if ((quoted_permalink = json_o_get(node, "quoted_status_permalink")) && quoted->type == json_object) {
+			quote_kort = json_o_strdup(quoted_permalink, "url");
+		}
 		txs_free(txs);
 	} else {
 		quoted = NULL;
@@ -749,7 +753,7 @@ static void expand_entities(char **text, const json_value *node, const json_valu
 			const char *full = json_o_str(v->u.array.values[i], "expanded_url");
 			char *pos, *new;
 
-			/* Skip if a required field is missing, if the t.co URL is not in fact 
+			/* Skip if a required field is missing, if the t.co URL is not in fact
 			   in the Tweet at all, or if the full-ish one *is* in it already
 			   (dupes appear, especially in streaming API). */
 			if (!kort || !disp || !(pos = strstr(*text, kort)) || strstr(*text, disp)) {
@@ -758,6 +762,7 @@ static void expand_entities(char **text, const json_value *node, const json_valu
 			if (quote_url && strstr(full, quote_url)) {
 				format = "%s<%s> [%s]%s";
 				disp = quote_text;
+				quote_used = TRUE;
 			}
 
 			*pos = '\0';
@@ -768,8 +773,17 @@ static void expand_entities(char **text, const json_value *node, const json_valu
 			*text = new;
 		}
 	}
+
+	if (quote_text && !quote_used) {
+		const char *url = quote_kort ?: quote_url;
+		char *new = g_strdup_printf("%s <%s>[%s]", *text, url, quote_text);
+		g_free(*text);
+		*text = new;
+	}
+
 	g_free(quote_text);
 	g_free(quote_url);
+	g_free(quote_kort);
 }
 
 /**
@@ -977,9 +991,9 @@ static void twitter_status_show(struct im_connection *ic, struct twitter_xml_sta
 	if (status->user == NULL || status->text == NULL) {
 		return;
 	}
-	
+
 	/* Check this is not a tweet that should be muted */
-	uid_str = g_strdup_printf("%" PRIu64, status->user->uid);
+	uid_str = g_strdup_printf("%" G_GUINT64_FORMAT, status->user->uid);
 
 	if (g_slist_find_custom(td->mutes_ids, uid_str, (GCompareFunc)strcmp)) {
 		g_free(uid_str);
@@ -1169,21 +1183,21 @@ static gboolean twitter_stream_handle_event(struct im_connection *ic, json_value
 		GSList *found;
 		char *uid_str;
 		ut = twitter_xt_get_user(target);
-		uid_str = g_strdup_printf("%" PRIu64, ut->uid);
+		uid_str = g_strdup_printf("%" G_GUINT64_FORMAT, ut->uid);
 		if (!(found = g_slist_find_custom(td->mutes_ids, uid_str,
 		                                  (GCompareFunc)strcmp))) {
 			td->mutes_ids = g_slist_prepend(td->mutes_ids, uid_str);
 		}
 		twitter_log(ic, "Muted user %s", ut->screen_name);
 		if (getenv("BITLBEE_DEBUG")) {
-			fprintf(stderr, "New mute: %s %"PRIu64"\n",
+			fprintf(stderr, "New mute: %s %"G_GUINT64_FORMAT"\n",
 			        ut->screen_name, ut->uid);
 		}
 	} else if (strcmp(type, "unmute") == 0) {
 		GSList *found;
 		char *uid_str;
 		ut = twitter_xt_get_user(target);
-		uid_str = g_strdup_printf("%" PRIu64, ut->uid);
+		uid_str = g_strdup_printf("%" G_GUINT64_FORMAT, ut->uid);
 		if ((found = g_slist_find_custom(td->mutes_ids, uid_str,
 		                                (GCompareFunc)strcmp))) {
 			char *found_str = found->data;
@@ -1193,7 +1207,7 @@ static gboolean twitter_stream_handle_event(struct im_connection *ic, json_value
 		g_free(uid_str);
 		twitter_log(ic, "Unmuted user %s", ut->screen_name);
 		if (getenv("BITLBEE_DEBUG")) {
-			fprintf(stderr, "New unmute: %s %"PRIu64"\n",
+			fprintf(stderr, "New unmute: %s %"G_GUINT64_FORMAT"\n",
 			        ut->screen_name, ut->uid);
 		}
 	}
@@ -1689,6 +1703,13 @@ void twitter_post_status(struct im_connection *ic, char *msg, guint64 in_reply_t
 		g_strdup_printf("%" G_GUINT64_FORMAT, in_reply_to)
 	};
 
+	if (set_getbool(&ic->acc->set, "in_korea") && !in_reply_to) {
+		g_free(args[3]);
+		args[2] = "place_id";
+		args[3] = g_strdup("c999e6a453e9ef72");
+		in_reply_to = 1;
+	}
+
 	twitter_http(ic, TWITTER_STATUS_UPDATE_URL, twitter_http_post, ic, 1,
 	             args, in_reply_to ? 4 : 2);
 	g_free(args[3]);
