@@ -440,7 +440,7 @@ static void cmd_account(irc_t *irc, char **cmd)
 		if (prpl == NULL) {
 			char *msg = explain_unknown_protocol(cmd[2]);
 			irc_rootmsg(irc, "Unknown protocol");
-			irc_rootmsg(irc, msg);
+			irc_rootmsg(irc, "%s", msg);
 			g_free(msg);
 			return;
 		}
@@ -594,7 +594,7 @@ static void cmd_account(irc_t *irc, char **cmd)
 			char *proto = set_getstr(&a->set, "_protocol_name");
 			char *msg = explain_unknown_protocol(proto);
 			irc_rootmsg(irc, "Unknown protocol `%s'", proto);
-			irc_rootmsg(irc, msg);
+			irc_rootmsg(irc, "%s", msg);
 			g_free(msg);
 		} else {
 			account_on(irc->b, a);
@@ -772,7 +772,7 @@ static void cmd_remove(irc_t *irc, char **cmd)
 
 	bu->ic->acc->prpl->remove_buddy(bu->ic, bu->handle, NULL);
 	nick_del(bu);
-	if (g_slist_find(irc->users, iu)) {
+	if (g_hash_table_contains(irc->nick_user_hash, iu->key)) {
 		bee_user_free(irc->b, bu);
 	}
 
@@ -818,7 +818,7 @@ static void cmd_rename(irc_t *irc, char **cmd)
 	iu = irc_user_by_name(irc, cmd[del ? 2 : 1]);
 
 	if (iu == NULL) {
-		irc_rootmsg(irc, "Nick `%s' does not exist", cmd[1]);
+		irc_rootmsg(irc, "Nick `%s' does not exist", cmd[del ? 2 : 1]);
 	} else if (del) {
 		if (iu->bu) {
 			bee_irc_user_nick_reset(iu);
@@ -1032,7 +1032,7 @@ static void cmd_set(irc_t *irc, char **cmd)
 static void cmd_blist(irc_t *irc, char **cmd)
 {
 	int online = 0, away = 0, offline = 0, ismatch = 0;
-	GSList *l;
+	GList *l, *users;
 	GRegex *regex = NULL;
 	GError *error = NULL;
 	char s[256];
@@ -1056,7 +1056,7 @@ static void cmd_blist(irc_t *irc, char **cmd)
 	}
 
 	if (error) {
-		irc_rootmsg(irc, error->message);
+		irc_rootmsg(irc, "%s", error->message);
 		g_error_free(error);
 	}
 
@@ -1073,7 +1073,10 @@ static void cmd_blist(irc_t *irc, char **cmd)
 		irc->root->last_channel = NULL;
 	}
 
-	for (l = irc->users; l; l = l->next) {
+	users = g_hash_table_get_values(irc->nick_user_hash);
+	users = g_list_sort(users, irc_user_cmp);
+
+	for (l = users; l; l = l->next) {
 		irc_user_t *iu = l->data;
 		bee_user_t *bu = iu->bu;
 
@@ -1119,6 +1122,8 @@ static void cmd_blist(irc_t *irc, char **cmd)
 		}
 	}
 
+	g_list_free(users);
+
 	irc_rootmsg(irc, "%d buddies (%d available, %d away, %d offline)", n_online + n_away + n_offline, n_online,
 	            n_away, n_offline);
 
@@ -1235,7 +1240,7 @@ static void cmd_plugins(irc_t *irc, char **cmd)
 	}
 #endif
 
-	irc_rootmsg(irc, "");
+	irc_rootmsg(irc, " ");
 
 	gstr = g_string_new(NULL);
 	prpls = get_protocols();
@@ -1341,13 +1346,19 @@ static void cmd_chat(irc_t *irc, char **cmd)
 		    set_setstr(&ic->set, "chat_type", "room") &&
 		    set_setstr(&ic->set, "account", cmd[2]) &&
 		    set_setstr(&ic->set, "room", room)) {
-			irc_rootmsg(irc, "Chatroom successfully added.");
+			irc_rootmsg(irc, "Chatroom successfully added, join with \002/join %s\002", channel);
 		} else {
 			if (ic) {
 				irc_channel_free(ic);
-			}
 
-			irc_rootmsg(irc, "Could not add chatroom.");
+				irc_rootmsg(irc, "Error adding chatroom.");
+			} else if (irc_channel_by_name(irc, channel)) {
+				irc_rootmsg(irc, "A channel named `%s' already exists. "
+				            "Join with \002/join %s\002 or see \002help channel\002 to modify it",
+				            channel, channel);
+			} else {
+				irc_rootmsg(irc, "Error creating channel for chatroom.");
+			}
 		}
 		g_free(channel);
 	} else if (g_strcasecmp(cmd[1], "list") == 0) {
@@ -1439,7 +1450,7 @@ void cmd_chat_list_finish(struct im_connection *ic)
 		ci = l->data;
 		topic = ci->topic ? ci->topic : "";
 
-		padded = str_pad_and_truncate(ci->title, title_len, "[...]");
+		padded = str_pad_and_truncate(ci->title ? ci->title : "", title_len, "[...]");
 		irc_rootmsg(irc, iformat, ++i, padded, topic);
 		g_free(padded);
 	}
@@ -1554,11 +1565,6 @@ static void cmd_transfer(irc_t *irc, char **cmd)
 	}
 }
 
-static void cmd_nick(irc_t *irc, char **cmd)
-{
-	irc_rootmsg(irc, "This command is deprecated. Try: account %s set display_name", cmd[1]);
-}
-
 /* Maybe this should be a stand-alone command as well? */
 static void bitlbee_whatsnew(irc_t *irc)
 {
@@ -1599,7 +1605,6 @@ command_t root_commands[] = {
 	{ "help",           0, cmd_help,           0 },
 	{ "identify",       0, cmd_identify,       0 },
 	{ "info",           1, cmd_info,           0 },
-	{ "nick",           1, cmd_nick,           0 },
 	{ "no",             0, cmd_yesno,          0 },
 	{ "plugins",        0, cmd_plugins,        0 },
 	{ "qlist",          0, cmd_qlist,          0 },
